/**
* user:  A1001
* fname: Aleksandar
* lname: Krastev
* task:  colorgraph
* score: 55.0
* date:  2017-11-25 10:40:29.761883
*/
//It's spelled "colour"
#include <iostream>
#include <cstdint>
#include <vector>

const uint32_t INF = 1e9;

struct NVertex {
	std::vector<NVertex*> edges;
	uint32_t comp;
	uint32_t corEdges;
	
	NVertex(): comp(INF) {}

	void dfs() {
		for (NVertex *v : edges) {
			if (v->comp == INF) {
				v->comp = comp;
				v->dfs();
			}
		}
	}
};

const uint32_t MAX_N = 251;

bool color[MAX_N][MAX_N];
uint32_t n;
uint32_t tR, tB;
uint32_t cR, cB;

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> n;
	for (uint32_t i = 0;i < n;i++) {
		for (uint32_t j = 0;j < n;j++) {
			std::cin >> color[i][j];
		}
	}
	std::cin >> tR >> tB;
}

NVertex rGraph[MAX_N];
NVertex bGraph[MAX_N];

void determine_state() {
	for (uint32_t i = 0;i < n;i++) {
		for (uint32_t j = 0;j < n;j++) {
			if (i == j) {
				continue;
			}
			if (color[i][j]) {
				bGraph[i].edges.push_back(&bGraph[j]);
			} else {
				rGraph[i].edges.push_back(&rGraph[j]);
			}
		}
	}

	uint32_t rComp = 0;
	uint32_t bComp = 0;
	for (uint32_t i = 0;i < n;i++) {
		if (bGraph[i].comp == INF) {
			bGraph[i].comp = bComp++;
			bGraph[i].dfs();
		}
		if (rGraph[i].comp == INF) {
			rGraph[i].comp = rComp++;
			rGraph[i].dfs();
		}
	}

	cR = rComp == 1;
	cB = bComp == 1;
}

NVertex *min_vertex(NVertex *graph, uint32_t c) {
	NVertex *minEdges = nullptr;
	for (uint32_t i = 0;i < n;i++) {
		NVertex *v = &graph[i];
		if (v->comp == c) {
			if (minEdges == nullptr) {
				minEdges = v;
			}
			if (v->corEdges < minEdges->corEdges) {
				minEdges = v;
			}
		}
	}
	return minEdges;
}

void add_edges(NVertex *graph) {
	std::vector<std::pair<uint32_t, uint32_t>> ans;
	uint32_t curComp = 0;

	for (uint32_t i = 0;i < n;i++) {
		if (graph[i].comp == 0) {
			curComp++;
			graph[i].corEdges = graph[i].edges.size();
		}
	}

	while (curComp < n) {
		NVertex *minEdges1 = min_vertex(graph, 0);

		uint32_t comp = INF;
		for (uint32_t i = 0;i < n;i++) {
			if (graph[i].comp != 0) {
				comp = graph[i].comp;
				break;
			}
		}
		NVertex *minEdges2 = min_vertex(graph, comp);
		ans.push_back(std::make_pair(minEdges1 - graph, minEdges2 - graph));
		minEdges1->corEdges++;
		minEdges2->corEdges++;

		for (uint32_t i = 0;i < n;i++) {
			if (graph[i].comp == comp) {
				graph[i].comp = 0;
				curComp++;
			}
		}
	}

	std::cout << ans.size() << "\n";
	for (auto i : ans) {
		std::cout << i.first + 1 << " " << i.second + 1 << "\n";
	}
}

struct FVertex;

struct FEdge {
	uint32_t cap;
	FVertex *to;

	uint32_t rInd;
	FEdge &rev();
};

struct FVertex {
	std::vector<FEdge> edges;
	bool visited;

	FVertex(): visited(false) {}
	void add_edge(FVertex *to);
	bool push_flow();
};

FEdge& FEdge::rev() {
	return to->edges[rInd];
}

void FVertex::add_edge(FVertex *to) {
	edges.push_back(FEdge());
	to->edges.push_back(FEdge());

	edges.back().to = to;
	edges.back().cap = 1;
	edges.back().rInd = to->edges.size() - 1;

	to->edges.back().to = this;
	to->edges.back().cap = 1;
	to->edges.back().rInd = edges.size() - 1;
}

FVertex fGraph[MAX_N];
void clear_used() {
	for (uint32_t i = 0;i < n;i++) {
		fGraph[i].visited = false;
	}
}
FVertex *sink;

bool FVertex::push_flow() {
	visited = true;
	if (sink == this) {
		return true;
	}

	for (FEdge &e : edges) {
		if ((e.cap == 0) || (e.to->visited)) {
			continue;
		}
		if (e.to->push_flow()) {
			e.cap--;
			e.rev().cap++;
			return true;
		}
	}
	return false;
}

uint32_t max_flow(FVertex *pSink) {
	sink = pSink;
	for (uint32_t i = 0;i < n;i++) {
		for (FEdge &e : fGraph[i].edges) {
			e.cap = e.rev().cap = 1;
		}
	}

	uint32_t ans = 0;
	while (fGraph[0].push_flow()) {
		ans++;
		clear_used();
	}
	clear_used();
	return ans;
}

void build_fgraph(NVertex *graph) {
	for (uint32_t i = 0;i < n;i++) {
		for (NVertex *e : graph[i].edges) { 
			uint32_t j = e - graph;
			if (i > j) {
				fGraph[i].add_edge(&fGraph[j]);
			}
		}
	}
}

void print_min_cut() {
	for (uint32_t i = 0;i < n;i++) {
		if (!fGraph[i].visited) {
			continue;
		}
		for (FEdge &e : fGraph[i].edges) {
			if (!e.to->visited) {
				uint32_t j = e.to - fGraph;
				std::cout << i + 1 << " " << j + 1 << "\n";
			}
		}
	}
}

/*
black and white
red and blue
I think they'll lok good on you
*/
void cut_edges(NVertex *graph) {
	build_fgraph(graph);
	uint32_t bestSink = 0;
	uint32_t bestCut = n * 2;

	for (uint32_t i = 1;i < n;i++) {
		uint32_t curFlow = max_flow(&fGraph[i]);
		if (curFlow < bestCut) {
			bestCut = curFlow;
			bestSink = i;
		}
	}

	std::cout << bestCut << "\n";

	max_flow(&fGraph[bestSink]);
	fGraph[0].push_flow();
	print_min_cut();
}

int main() {
	input();
	determine_state();

	//std::cerr << cR << " " << cB << "\n";

	if ((cR == tR) && (tB == cB)) {
		std::cout << 0 << "\n";
		return 0;
	} 
	if ((n == 3) && (tR == 1) && (tB == 1)) {
		std::cout << -1 << "\n";
		return 0;
	}
	if ((tR == 0) && (tB == 0)) {
		std::cout << -1 << "\n";
		return 0;
	}

	if ((tR == 1) && (tB == 1)) {
		if (tR == 1) {
			add_edges(rGraph);
		} else {
			add_edges(bGraph);
		}
	} else {
		if (tR == 0) {
			cut_edges(rGraph);
		} else {
			cut_edges(bGraph);
		}
	}
}
