/**
* user:  A1005
* fname: Aleksandar
* lname: Ivanov
* task:  colorgraph
* score: 0.0
* date:  2017-11-25 11:59:42.682763
*/
#include <vector>
#include <algorithm>
#include <iostream>
#include <cstdio>
#include <queue>
#include <set>
using namespace std;

int n, a, b;
vector<vector<int>> redGr, blueGr;
vector<vector<bool>> isRed;
vector<pair<int, int>> redEdges, blueEdges;

bool equal(int x, int y, int x1, int y1)
{
    return (x==x1 && y==y1) || (x==y1 && y==x1);
}

bool is_ap(const vector<vector<int>> gr, int x, int y)
{
    vector<bool> used(gr.size(), false);
    queue<int> bfs;
    bfs.push(x);
    used[x]=true;
    int usedCount =1;

    while(!bfs.empty())
    {
        int current = bfs.front();
        bfs.pop();

        for(int next : gr[current])
        {
            if(used[next])continue;
            if(equal(current, next, x, y))continue;

            used[next]=true;
            bfs.push(next);
            usedCount++;
        }
    }

    return usedCount == gr.size()-1;
}

bool is_connected(const vector<vector<int>> &gr)
{
    return is_ap(gr, 1, -1);
}

vector<pair<int, int>> get_edges(const vector<vector<int>> &gr)
{
    vector<pair<int, int>> edges;
    for(int i=1;i<gr.size();i++)
    {
        for(int j : gr[i])if(i<j)
        {
            edges.emplace_back(i, j);
        }
    }
    return std::move(edges);
}

vector<pair<int, int>> find_aps(const vector <vector<int>> &gr)
{
    vector<pair<int, int>> aps;

    for(int i=1;i<gr.size();i++)
    {
        for(int j : gr[i])if(j>i)
        {
            if(is_ap(gr, i, j))aps.emplace_back(i, j);
        }
    }

    return std::move(aps);
}

bool unify(
    vector<vector<int>> &unions,
    vector<int> &unionOf,
    int u1, int u2
)
{
    if(unions[u1].size()>unions[u2].size())swap(u1, u2);
    if(unions[u1].empty())return false;

    for(int x : unions[u1])
    {
        unions[u2].push_back(x);
        unionOf[x]=u2;
    }

    unions[u1].clear();
    unions[u1].shrink_to_fit();
    return true;
}

vector<pair<int, int>> get_mst(int n, const vector<pair<int, int>> &included, const vector<pair<int, int>> &edges, const set<pair<int, int>> &forbidden)
{
    int unionCount = n;
    vector<pair<int, int>> mst;
    vector<vector<int>> unions(n+1);
    vector<int> unionOf(n+1, -1);

    for(int i=1;i<=n;i++)
    {
        unionOf[i]=i;
        unions[i].push_back(i);
    }

    for(auto e : included)
    {
        if(unify(unions, unionOf, unionOf[e.first], unionOf[e.second]))
        {
            unionCount--;
        }
    }

    for(pair<int, int> edge : edges)
    {
        if(unionCount==1)break;
        if(forbidden.find(edge)!=forbidden.end())continue;

        int x=edge.first, y=edge.second;
        if(unionOf[x]!=unionOf[y])
        {
            if(unify(unions, unionOf, unionOf[x], unionOf[y]))
            {
                unionCount--;
            }
            mst.push_back(edge);
        }

    }

    return std::move(mst);
}

bool is_possible(int n, bool red, bool blue)
{
    bool one = red!=blue;
    bool two = red&&blue;

    if(one)return true;

    if(n==3)
    {
        //only one can be connected
        return !(red&&blue);
    }
    else
    {
        return true;
    }
}

void read()
{
    cin>>n;
    redGr.resize(n+1);
    blueGr.resize(n+1);
    isRed.resize(n+1, vector<bool>(n+1, false));
    for(int i=1;i<=n;i++)
    {
        for(int j=1;j<=n;j++)
        {
            int x;
            cin>>x;
            isRed[i][j]=(x==0);
            if(x==1)
            {
                blueGr[i].push_back(j);
                blueGr[j].push_back(i);
                blueEdges.emplace_back(i, j);
                blueEdges.emplace_back(j, i);
            }
            else
            {
                redGr[i].push_back(j);
                redGr[j].push_back(i);
                redEdges.emplace_back(i, j);
                redEdges.emplace_back(j, i);
            }
        }
    }
    cin>>a>>b;
}

void fail()
{
    puts("-1");
    exit(0);
}

void solve()
{
    bool redConnected = is_connected(redGr);
    bool blueConnected = is_connected(blueGr);

    if(redConnected == a && blueConnected == b)
    {
        puts("0");
        exit(0);
    }

    if(redConnected && !a)//gotta disconnect reds
    {
vector<pair<int, int>> redAPs = find_aps(redGr);
            if(!redAPs.empty())
            {
                puts("1");
                printf("%d %d\n", redAPs[0].first, redAPs[0].second);
            }
    }

    if(blueConnected && !b)//gotta disconnect blues
    {

        vector<pair<int, int>> blueAPs = find_aps(blueGr);
        if(!blueGr.empty())
        {
            puts("1");
            printf("%d %d\n", blueAPs[0].first, blueAPs[0].second);
        }
    }

    if(!is_possible(n, a, b))fail();

    vector<pair<int, int>> toChange;

    if(blueConnected && a)//we need to connect red now
    {
        vector<pair<int, int>> blueAPs=find_aps(blueGr);
        set<pair<int, int>> blueApSet{blueAPs.begin(), blueAPs.end()};
        vector<pair<int, int>> redMst = get_mst(n, redEdges, blueEdges, blueApSet);
        toChange=std::move(redMst);
    }

    if(redConnected && b)//we need to connect blue now
    {
        vector<pair<int, int>> redAPs=find_aps(redGr);
        set<pair<int, int>> redApSet{redAPs.begin(), redAPs.end()};
        vector<pair<int, int>> blueMst=get_mst(n, blueEdges, redEdges, redApSet);
        toChange=std::move(blueMst);
    }

    printf("%d\n", (int)toChange.size());
    for(auto &x : toChange)printf("%d %d\n", x.first, x.second);
    putchar('\n');

    puts("lolo");
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    read();
    solve();
}


/**
4
1 0 0 0
0 0 1 0
0 1 1 0
0 0 0 0
0 1

3
0 1 1
1 0 0
1 0 0
0 1

3
0 1 1
1 0 0
1 0 0
1 1
*/
